package com.example.damien.whichfingers;

import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbConstants;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.util.Log;
import android.widget.Toast;

import java.util.HashMap;
import java.util.Iterator;

/**
 * Created by damien on 12/10/17.
 */

public class WhichFingersHID {

    private static final String TAG = "WhichFingersHID";
    private UsbManager mManager;
    private UsbDevice  mDevice;
    private UsbDeviceConnection mDeviceConnection;
    private UsbInterface mInterface;
    private UsbEndpoint mEndpointIn;
    private Context context;

    private byte piezoValues[][] = new byte[5][10000];
    private int piezoIdx = 0;

    public WhichFingersHID(Context context) {
        this.context = context;
    }

    private int getPiezoIdx(int idx ) {
        while (idx < 0) idx += piezoValues[0].length;
        return idx % piezoValues[0].length;
    }

    public byte getPiezoVal(int piezoId, int idx) {
        return piezoValues[piezoId][getPiezoIdx(idx)];
    }

    public int getCurrentIdx() {
        return piezoIdx;
    }

    public UsbDevice connectToDevice(BroadcastReceiver mUsbReceiver) {

        mManager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
        HashMap<String, UsbDevice> deviceList = mManager.getDeviceList();
        Iterator<UsbDevice> deviceIterator = deviceList.values().iterator();

        while (deviceIterator.hasNext()) {
            mDevice = deviceIterator.next();
            Log.d(TAG,"Model: " + mDevice.getDeviceName());
            Log.d(TAG,"ID: " + mDevice.getDeviceId());
            Log.d(TAG,"Class: " + mDevice.getDeviceClass());
            Log.d(TAG,"Protocol: " + mDevice.getDeviceProtocol());
            Log.d(TAG,"Vendor ID " + mDevice.getVendorId());
            Log.d(TAG,"Product ID: " + mDevice.getProductId());
            Log.d(TAG,"Interface count: " + mDevice.getInterfaceCount());
            Log.d(TAG,"---------------------------------------");
            // Get interface details
            for (int index = 0; index < mDevice.getInterfaceCount(); index++) {
                UsbInterface mUsbInterface = mDevice.getInterface(index);
                Log.d(TAG,"  *****     *****");
                Log.d(TAG,"  Interface index: " + index);
                Log.d(TAG,"  Interface ID: " + mUsbInterface.getId());
                Log.d(TAG,"  Inteface class: " + mUsbInterface.getInterfaceClass());
                Log.d(TAG,"  Interface protocol: " + mUsbInterface.getInterfaceProtocol());
                Log.d(TAG,"  Endpoint count: " + mUsbInterface.getEndpointCount());
                // Get endpoint details
                for (int epi = 0; epi < mUsbInterface.getEndpointCount(); epi++) {
                    UsbEndpoint mEndpoint = mUsbInterface.getEndpoint(epi);
                    Log.d(TAG,"    ++++   ++++   ++++");
                    Log.d(TAG,"    Endpoint index: " + epi);
                    Log.d(TAG,"    Attributes: " + mEndpoint.getAttributes());
                    Log.d(TAG,"    Direction: " + mEndpoint.getDirection());
                    Log.d(TAG,"    Number: " + mEndpoint.getEndpointNumber());
                    Log.d(TAG,"    Interval: " + mEndpoint.getInterval());
                    Log.d(TAG,"    Packet size: " + mEndpoint.getMaxPacketSize());
                    Log.d(TAG,"    Type: " + mEndpoint.getType());
                }
            }
        }
        Log.d(TAG," No more devices connected.");

        if (mDevice == null) {
            Toast.makeText(context, "Device not connected !", Toast.LENGTH_LONG).show();
        } else {
            if (!mManager.hasPermission(mDevice)) {
                PendingIntent pi = PendingIntent.getBroadcast(context, 42, new Intent("com.example.damien.whichfingers.USB_PERMISSION"), 0);
                context.registerReceiver(mUsbReceiver, new IntentFilter("com.example.damien.whichfingers.USB_PERMISSION"));
                mManager.requestPermission(mDevice, pi);
            } else {
                mUsbReceiver.onReceive(context, new Intent("com.example.damien.whichfingers.USB_PERMISSION"));
            }
        }
        return null;
    }

    public void run() {
        while (mEndpointIn != null && mDeviceConnection != null) {
            byte[] recordIn = new byte[mEndpointIn.getMaxPacketSize()];
            int receivedLength = mDeviceConnection.bulkTransfer(mEndpointIn, recordIn,
                    recordIn.length, 1000);

            if (receivedLength == 7) {
                piezoIdx = (piezoIdx + 1) % piezoValues[0].length;
                String str = "Received " + receivedLength + " bytes :";
                for (int i = 1; i < 6; ++i) {
                    piezoValues[i - 1][piezoIdx] = recordIn[i];
                    str += " " + recordIn[i];
                }
                //Log.d(TAG,str);
            }
        }
    }

    public void initializeCommunication() {

        mInterface = findHidInterface();

        if (mManager != null && mDevice != null && mInterface != null) {
            mDeviceConnection = mManager.openDevice(mDevice);

            if (mDeviceConnection != null) {
                if (mDeviceConnection.claimInterface(mInterface, true)) {
                    UsbEndpoint usbEndpointOut = null;
                    UsbEndpoint usbEndpointIn = null;

                    // Look for Interrupt endpoints
                    for (int i = 0; i < mInterface.getEndpointCount(); i++) {
                        UsbEndpoint usbEndpoint = mInterface.getEndpoint(i);

                        if (usbEndpoint.getType() == UsbConstants.USB_ENDPOINT_XFER_INT) {
                            if (usbEndpoint.getDirection() == UsbConstants.USB_DIR_OUT) {
                                usbEndpointOut = usbEndpoint;

                            } else {
                                usbEndpointIn = usbEndpoint;
                            }
                        }
                    }

                    if (usbEndpointOut == null || usbEndpointIn == null) {
                        Log.d(TAG,"No USB endpoints found.");
                    }
                    mEndpointIn = usbEndpointIn;
                } else {
                    mDeviceConnection.close();
                    Log.d(TAG,"USB claim interface failed.");
                }
            } else {
                Log.d(TAG,"USB open device failed.");
            }
        } else {
            Log.d(TAG,"Some essential USB variables were null.");
        }
    }

    /**
     * Find the HID interface.
     *
     * @return
     *      Return the HID interface if found, otherwise null.
     */
    private UsbInterface findHidInterface() {
        if (mDevice != null) {
            final int interfaceCount = mDevice.getInterfaceCount();

            for (int interfaceIndex = 0; interfaceIndex < interfaceCount; interfaceIndex++) {
                UsbInterface usbInterface = mDevice.getInterface(interfaceIndex);

                // Can add UsbInterface.getInterfaceSubclass() and
                // UsbInterface.getInterfaceProtocol() for more specifics.

                if (usbInterface.getInterfaceClass() == UsbConstants.USB_CLASS_HID) {
                    return usbInterface;
                }
            }
            Log.d(TAG,"HID interface not found.");
        }
        return null;
    }
}
